@@ -5,7 +5,7 @@ oer - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2003 EQU <equ@equnet.org>
+Copyright (C) 2000-2004 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -101,7 +101,7 @@ extern int uptime(time_t *, time_t *);
 extern int deladmin(char *);
 extern int deladminmask(char *, char *);
 extern int deluser(struct channel *, char *);
-extern int delusermask(struct channel *, char *, char *);
+extern int delusermasks(struct channel *, char *, char *, int);
 extern int deltrusted(char *);
 extern int delservice(char *);
 extern int delchannel(char *);
@@ -127,6 +127,7 @@ extern int ischanopless(struct channel *);
 extern int joininprogress(void);
 extern int getmmodecount(struct channel *);
 extern int gettimedcount(void);
+extern int cleanautheds(struct channel *, int);
 extern unsigned int getrandom(unsigned int);
 extern long getfilesize(char *);
 extern struct authed *addnewauthed(struct channel *, time_t, struct botuser *, char *);
@@ -235,7 +236,6 @@ extern void quit(void);
 extern void parsectcp(char *, char *, int, char *);
 extern void nstats(struct channel *, char *, int, char *);
 extern void wall(struct channel *, char *, char *, char *);
-extern void cleanautheds(struct channel *, int);
 extern void listchanusers(void);
 extern void timed_del(struct timed *);
 extern void mmode_del(struct channel *, struct mmode *);
