@@ -5,7 +5,7 @@ oer - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2003 EQU <equ@equnet.org>
+Copyright (C) 2000-2004 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -42,6 +42,7 @@ extern int completeban(char *, char *, int);
 extern int issimilarstring(char *, char *);
 extern int countchars(char *, char);
 extern int countnchars(char *, char);
+extern int parseprotocolmessage(char *, char *, int, char *, int, char *, int, char *, int);
 extern unsigned int longestword(char *);
 extern unsigned int isgoodquote(char *);
 extern char *mybasename(char *);
@@ -54,6 +55,7 @@ extern void safequote(char *);
 extern void oer_debug(int, const char *, ...);
 extern void secondstostring(int, char *, int);
 extern void sortstring(char *);
+extern void filteroutchanmodes(char *, char *, char *, int);
 
 #ifndef HAVE_SNPRINTF
 extern int snprintf(char *, size_t, const char *, ...);
