@@ -5,7 +5,7 @@ oer - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2003 EQU <equ@equnet.org>
+Copyright (C) 2000-2004 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -51,6 +51,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <unistd.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <pwd.h>
 #include <signal.h>
 
@@ -59,6 +60,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/stat.h>
 #include <sys/socket.h>
 
+#ifdef HAVE_UNAME
+  #include <sys/utsname.h>
+#endif
+
 #ifdef TIME_WITH_SYS_TIME
   #include <sys/time.h>
   #include <time.h>
@@ -187,8 +192,8 @@ enum ESTABLISHCONNECTION_RETURN_VALUES {
 enum SERVERCONNECTION_RETURN_VALUES {
 	OER_SERVERCONNECTION_ERR_SOCKET,
 	OER_SERVERCONNECTION_ERR_GETHOSTNAME,
-	OER_SERVERCONNECTION_ERR_GETHOSTBYNAME,
-	OER_SERVERCONNECTION_ERR_GETHOSTBYNAME_RETRY,
+	OER_SERVERCONNECTION_ERR_GETADDRINFO,
+	OER_SERVERCONNECTION_ERR_GETADDRINFO_RETRY,
 	OER_SERVERCONNECTION_ERR_GETSERVER_RETRY,
 	OER_SERVERCONNECTION_ERR_BIND,
 	OER_SERVERCONNECTION_ERR_CONNECT,
@@ -342,6 +347,7 @@ enum OER_FLAGS_TYPE {
 #define OER_BURST_JOINS_LIMIT OER_JOINS / 2
 #define OER_DELAY_BETWEEN_REJOINS 10
 #define OER_DELAY_BETWEEN_REJOINS_CYCLE 1
+#define OER_DELAY_CHANMODES_AFTER_JOIN 10
 #define OER_AUTO_REJOIN_TIME 3
 #define OER_QUOTE_INTERVAL 300
 #define OER_ADVERT_INTERVAL 300
@@ -414,7 +420,7 @@ enum OER_FLAGS_TYPE {
 #define OER_SERVER_IS_STONED "stoned server"
 #define OER_INTERNAL_DYNAMIC_ADMIN "*INTDYN*"
 #define OER_REALNAME "http://oer.equnet.org"
-#define OER_VERSION "oer version 1.0-63"
+#define OER_VERSION "oer version 1.0-64"
 #define OER_COPYRIGHT1 "Copyright (C) 2000-2003 EQU <equ@equnet.org>"
 #define OER_COPYRIGHT2 "oer comes with ABSOLUTELY NO WARRANTY; for details see the\n\
 GNU General Public License (COPYING) provided with this distribution.\n\
@@ -643,6 +649,8 @@ struct channel {
 	int requestop_now;
 	int seencounter;
 	int lastcounter;
+	int chankeyisset;
+	int chanlimitisset;
 	time_t rejoin_at;
 	time_t last_quote;
 	time_t join_ts;
