@@ -5,7 +5,7 @@ oer - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2003 EQU <equ@equnet.org>
+Copyright (C) 2000-2004 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,6 +35,7 @@ int checkauth(struct channel *, char *, char *, int, int);
 void parseirc(char *);
 void parsecommand(char *, char *, char *, char *);
 void parsemsg(char *, char *, char *, char *);
+void parsenotice(char *, char *, char *, char *);
 
 void parseirc(char *serveroutput)
 {
@@ -52,6 +53,7 @@ void parseirc(char *serveroutput)
 	char prefix[STRINGLEN + 1];
 	char command[TINYSTRINGLEN + 1];
 	char params[STRINGLEN + 1];
+	char trailing[WRITE_BUFFER_LENGTH + 1];
 	char outstring[STRINGLEN + 1];
 	char outstring2[WRITE_BUFFER_LENGTH + 1];
 	char outstring3[WRITE_BUFFER_LENGTH + 1];
@@ -83,24 +85,22 @@ void parseirc(char *serveroutput)
 		/* must be a command */
 		return;
 	}
-	ppos = 0;
-	if((ppos = parse(serveroutput, ppos, " ", prefix, STRINGLEN, 0)) < 0) {
-		return;
-	}
-	if((ppos = parse(serveroutput, ppos, " ", command, TINYSTRINGLEN, 0)) < 0) {
-		return;
-	}
-	if((ppos = parse(serveroutput, ppos, " ", params, STRINGLEN, 0)) < 0) {
-		return;
-	}
+	ppos = parseprotocolmessage(serveroutput, prefix, STRINGLEN, command, TINYSTRINGLEN, params, STRINGLEN, trailing, WRITE_BUFFER_LENGTH);
 	/* the rest of the serveroutput is serveroutput + ppos */
 	restptr = serveroutput + ppos;
+#ifdef DEBUG
+	oer_debug(OER_DEBUG_NOISE, "parseirc->%s\n", prefix);
+	oer_debug(OER_DEBUG_NOISE, "parseirc->%s\n", command);
+	oer_debug(OER_DEBUG_NOISE, "parseirc->%s\n", params);
+	oer_debug(OER_DEBUG_NOISE, "parseirc->%s\n", trailing);
+	oer_debug(OER_DEBUG_NOISE, "parseirc->%s\n", restptr);
+#endif
 	/* the rest of the line depends on the command/numeric */
 	if(!strcasecmp(command, "001")) {
 		/* we have registered & are connected to the server */
                 mystate->current_server->registered.done = 1;
 		/* store the real server name so that sstats shows it correctly */
-		strncpy(mystate->current_server->serverhost_r, prefix + 1, HOSTLEN);
+		strncpy(mystate->current_server->serverhost_r, prefix, HOSTLEN);
 		/* force a userhost on our nick to find out our hostname */
 		snprintf(timed_str, WRITE_BUFFER_LENGTH, "USERHOST %s", mystate->nick);
                 timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, timed_str);
@@ -159,11 +159,9 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "302")) {
-		/* first char is ':' */
-		restptr++;
 		restpos = 0;
 		while(1) {
-			nrestpos = parse(restptr, restpos, " ", outstring, STRINGLEN, 0);
+			nrestpos = parse(trailing, restpos, " ", outstring, STRINGLEN, 0);
 			if(nrestpos == restpos || nrestpos < 0) {
 				break;
 			}
@@ -247,12 +245,8 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "315")) {
-		/* end of WHO, get bans */
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-		if((this = getchptr(channel)) == NULL) {
+		/* end of WHO, next query bans */
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
 		this->whoquery = 1;
@@ -272,10 +266,9 @@ void parseirc(char *serveroutput)
 	if(!strcasecmp(command, "319")) {
 		/* returned for WHOIS */
 		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0)) < 0) {
+		if((restpos = parse(trailing, restpos, " ", temp_nick, STRINGLEN, 0)) < 0) {
 			return;
 		}
-		restpos++;
 		while(1) {
 			nrestpos = parse(restptr, restpos, " ", channel, CHANLEN, 0);
 			if(nrestpos == restpos || nrestpos < 0) {
@@ -305,6 +298,35 @@ void parseirc(char *serveroutput)
 		}
 		return;
 	}
+	if(!strcasecmp(command, "324")) {
+		if((this = getchptr(trailing)) == NULL) {
+			return;
+		}
+		/* channel mode(s) */
+		if(getmodepol(restptr, 'k') == '+') {
+			/* avoid 467 */
+			this->chankeyisset = 1;
+		} else if(getmodepol(restptr, 'l') == '+') {
+			if(getmodepol(this->mode, 'l') == '+') {
+				/* forced limit, reset to the limit stored */
+				this->chanlimitisset = 0;
+			} else {
+				/* get the dynamicly set limit */
+				tpos = 0;
+				if((tpos = parse(restptr, tpos, " ", mode, CHANLEN, 0)) < 0) {
+					return;
+				}
+				/* limit comes first */
+				strncpy(this->limit, restptr + tpos, CHANLEN);
+				this->chanlimitisset = 0;
+			}
+		}
+		return;
+	}
+	if(!strcasecmp(command, "329")) {
+		/* channel creation time */
+		return;
+	}
 	if(!strcasecmp(command, "330")) {
 		/* "authed as", this is the response to WHOIS */
 		return;
@@ -312,11 +334,7 @@ void parseirc(char *serveroutput)
 	if(!strcasecmp(command, "331")) {
 		/* the channel has no topic set, returned to a TOPIC command
 		   without params (for a channel which has no topic) */
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
                 if(haschanflags(this, "T")) {
@@ -330,11 +348,7 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "332")) {
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
                 if(haschanflags(this, "T")) {
@@ -348,8 +362,7 @@ void parseirc(char *serveroutput)
 			   ours, we reset it to our's */
 			return;
 		}
-		/* the topic is at restptr + restpos + 1 (first char is ':') */
-		gettopic(this, restptr + restpos + 1);
+		gettopic(this, restptr);
 		return;
 	}
 	if(!strcasecmp(command, "333")) {
@@ -361,10 +374,10 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "352")) {
 		/* returned by the WHO query initiated in isjoined() */
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
+		restpos = 0;
 		if((restpos = parse(restptr, restpos, " ", user, USERHOSTLEN, 0)) < 0) {
 			return;
 		}
@@ -380,9 +393,6 @@ void parseirc(char *serveroutput)
 		if((restpos = parse(restptr, restpos, " ", mode, CHANLEN, 0)) < 0) {
 			return;
 		}
-		if((this = getchptr(channel)) == NULL) {
-			return;
-		}
 		snprintf(userhost, USERHOSTLEN, "%s@%s", user, host);
 		setuserhost(temp_nick, userhost);
 		if(index(mode, (int)'@')) {
@@ -399,9 +409,6 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "353")) {
 		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", mode, CHANLEN, 0)) < 0) {
-			return;
-		}
 		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
 			return;
 		}
@@ -411,16 +418,15 @@ void parseirc(char *serveroutput)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parseirc(353)->%s\n", this->name);
 #endif
+		if(restptr[restpos] == ':') {
+			restpos++;
+		}
 		while(1) {
 			nrestpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0);
 			if(nrestpos == restpos || nrestpos < 0) {
 				break;
 			}
 			restpos = nrestpos;
-			if(temp_nick[0] == ':') {
-				strncpy(temp_nick2, temp_nick + 1, STRINGLEN);
-				strncpy(temp_nick, temp_nick2, STRINGLEN);
-			}
 #ifdef DEBUG
 			oer_debug(OER_DEBUG_INFO, "parseirc(353)->%s\n", temp_nick);
 #endif
@@ -445,10 +451,7 @@ void parseirc(char *serveroutput)
 	if(!strcasecmp(command, "366")) {
 		/* 366 is sent after /NAMES, means the join is complete */
 		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
 #ifdef DEBUG
@@ -459,14 +462,11 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "367")) {
 		/* channel ban returned from MODE-query */
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
 		/* ban mask */
+		restpos = 0;
 		if((restpos = parse(restptr, restpos, " ", userhost, USERHOSTLEN, 0)) < 0) {
 			return;
 		}
@@ -510,11 +510,7 @@ void parseirc(char *serveroutput)
 		oer_debug(OER_DEBUG_INFO, "parseirc(405)->%s\n", restptr);
 #endif
 		/* ERR_TOOMANYCHANNELS */
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
 		/* reset joining flag, we will try again in OER_DELAY_BETWEEN_REJOINS seconds */
@@ -529,10 +525,6 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "432") || !strcasecmp(command, "433")) {
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0)) < 0) {
-			return;
-		}
 		/* check if we are still registering */
                 if(!mystate->current_server->registered.done) {
 			mystate->current_server->registered.nick = 0;
@@ -544,7 +536,7 @@ void parseirc(char *serveroutput)
 				mystate->use_altnick = 0;
 			} else {
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, temp_nick, mystate->altnick);
+				oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, trailing, mystate->altnick);
 #endif
 				strncpy(mystate->nick, mystate->altnick, NICKLEN);
 				mystate->use_altnick = 1;
@@ -555,14 +547,7 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "437")) {
 		/* ERR_UNAVAILRESOURCE, handle both cases */
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-#ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(%s)->%s\n", command, channel);
-#endif
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(trailing)) == NULL) {
 			/* nick unavailable, check if we are still registering */
 			if(!mystate->current_server->registered.done) {
 				mystate->current_server->registered.nick = 0;
@@ -574,7 +559,7 @@ void parseirc(char *serveroutput)
 					mystate->use_altnick = 0;
 				} else {
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, channel, mystate->altnick);
+					oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, trailing, mystate->altnick);
 #endif
 					strncpy(mystate->nick, mystate->altnick, NICKLEN);
 					mystate->use_altnick = 1;
@@ -591,16 +576,10 @@ void parseirc(char *serveroutput)
                 return;
         }
 	if(!strcasecmp(command, "439")) {
-#ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(439)->%s\n", restptr);
-#endif
+		if((this = getchptr(trailing)) == NULL) {
+			return;
+		}
 		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-		if((this = getchptr(channel)) == NULL) {
-			return;
-		}
 		if((ptr = strstr(restptr + restpos, "wait")) == NULL) {
 			return;
 		}
@@ -612,7 +591,7 @@ void parseirc(char *serveroutput)
 		if(this->joining && !this->joined && i > 0) {
 			/* reset join status, retry when the server will accept our join */
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(439)->trying to join channel again in %d seconds\n", i);
+			oer_debug(OER_DEBUG_INFO, "parseirc(439)->trying to join channel %s again in %d seconds\n", this->name, i);
 #endif
 			this->joining = 0;
 			this->rejoin_at = mystate->now + i;
@@ -639,14 +618,7 @@ void parseirc(char *serveroutput)
 	/* couldn't join channel, reset joining flag */
 	if(!strcasecmp(command, "471") || !strcasecmp(command, "473") \
 	   || !strcasecmp(command, "474") || !strcasecmp(command, "475")) {
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
-			return;
-		}
-#ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(%s)->%s\n", command, channel);
-#endif
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
 		this->rejoin_at = mystate->now + OER_DELAY_BETWEEN_REJOINS;
@@ -664,9 +636,6 @@ void parseirc(char *serveroutput)
 	if(!strcasecmp(command, "484")) {
 		/* undernet extension, protected user */
 		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0)) < 0) {
-			return;
-		}
 		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
 			return;
 		}
@@ -674,9 +643,9 @@ void parseirc(char *serveroutput)
 			return;
 		}
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(484)->nick %s on %s is a IRC operator\n", temp_nick, this->name);
+		oer_debug(OER_DEBUG_INFO, "parseirc(484)->nick %s on %s is a IRC operator\n", trailing, this->name);
 #endif
-		changeuser(this, temp_nick, 1, -1, -1, -1);
+		changeuser(this, trailing, 1, -1, -1, -1);
 		return;
 	}
 	if(!strcasecmp(command, "513")) {
@@ -689,26 +658,20 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "JOIN")) {
-		/* build the necessary strings */
-		if(!strncasecmp(params, ":", 1)) {
-			strncpy(channel, params + 1, CHANLEN);
-		} else {
-			/* IRCNET doesn't put the preceeding ":" on
-			   netjoins */
-			strncpy(channel, params, CHANLEN);
-		}
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(params)) == NULL) {
 			return;
 		}
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		if(isme(temp_nick)) {
-			/* we are not interested in our own joins */
+			/* we are not interested in our own joins, parse channel mode(s) */
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s", this->name);
+			timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, timed_str);
 			return;
 		}
-		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
+		strncpy(userhost, prefix + tpos, USERHOSTLEN);
 		if(userjoined(this, temp_nick, userhost, 0, 0, 0, isfriend(this, temp_nick, userhost)) == NULL) {
 #ifdef DEBUG
 			oer_debug(OER_DEBUG_INFO, "parseirc(JOIN)->userjoined() returned NULL\n");
@@ -789,7 +752,7 @@ void parseirc(char *serveroutput)
 		}
 		gettimeofday(&tv2, NULL);
 		snprintf(outstring3, HUGESTRINGLEN, "\"%s\" (c) %s", outstring2, temp_nick2);
-		sendreply(channel, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring3);
+		sendreply(this->name, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring3);
 		this->last_quote = mystate->now;
 		snprintf(outstring3, HUGESTRINGLEN, "%ld secs and %ld usecs", tv2.tv_sec - tv1.tv_sec, ABS(tv2.tv_usec - tv1.tv_usec));
 		if(tv2.tv_sec == tv1.tv_sec) {
@@ -807,21 +770,19 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "PRIVMSG")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
-		strncpy(channel, params, CHANLEN);
-		/* the message starts at pos + 1 ... */
-		parsemsg(temp_nick, userhost, channel, restptr + 1);
+		strncpy(userhost, prefix + tpos, USERHOSTLEN);
+		parsemsg(temp_nick, userhost, params, trailing);
 		return;
 	}
 	if(!strcasecmp(command, "QUIT")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
+		strncpy(userhost, prefix + tpos, USERHOSTLEN);
 		/* quit doesn't provide the channel, we have to check all channels */
 		for(this = mystate->channels; this != NULL; this = this->next) {
 			if(!isonchan(this, temp_nick)) {
@@ -851,16 +812,10 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "PART")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
-		/* ircu PART behaviour has changed, why? */
-                if(params[0] == ':') {
-                        strncpy(channel, params + 1, CHANLEN);
-                } else {
-                        strncpy(channel, params, CHANLEN);
-                }
+		strncpy(userhost, prefix + tpos, USERHOSTLEN);
                 if((this = getchptr(channel)) == NULL) {
                         return;
                 }
@@ -883,9 +838,9 @@ void parseirc(char *serveroutput)
 		initmmodesfornick(this, temp_nick);
 		if(this->nickcount == 1 && !this->i_am_op) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(PART)->cycling %s to gain ops\n", channel);
+			oer_debug(OER_DEBUG_INFO, "parseirc(PART)->cycling %s to gain ops\n", this->name);
 #endif
-			snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s", channel);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s", this->name);
 			timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, timed_str);
 		}
 		listchanusers();
@@ -893,21 +848,21 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "NICK")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
+		strncpy(userhost, prefix + tpos, USERHOSTLEN);
 		if(isme(temp_nick)) {
 			/* oer changed it's nick either because it was 
 			 asked to or it wasn't the primary nick and
 			global flag "g" was set */
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "nickchange->I changed my nick from %s to %s\n", temp_nick, params + 1);
+			oer_debug(OER_DEBUG_INFO, "nickchange->I changed my nick from %s to %s\n", temp_nick, params);
 #endif
-			strncpy(mystate->nick, params + 1, NICKLEN);
+			strncpy(mystate->nick, params, NICKLEN);
 		}
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "nickchange->%s is now known as %s, changing on all channels\n", temp_nick, params + 1);
+		oer_debug(OER_DEBUG_INFO, "nickchange->%s is now known as %s, changing on all channels\n", temp_nick, params);
 #endif
 		/* if user changes nick while we are doing HOSTNAME queries
 		   on channel join, we will end up with userhost = NULL for the user */
@@ -921,14 +876,14 @@ void parseirc(char *serveroutput)
 				/* if a user tries to avoid getting kicked
 				   on !lock by changing nick, we will adapt */
 				changetobek(this, temp_nick, 0);
-				if(this->i_am_op && checkforbadnick(this, params + 1, userhost)) {
+				if(this->i_am_op && checkforbadnick(this, params, userhost)) {
 					changetobek(this, temp_nick, 1);
 				}
 				if(this->i_am_op && checkfornickflood(this, temp_nick, userhost)) {
 					changetobek(this, temp_nick, 1);
 				}
-				nickchange(this, temp_nick, params + 1);
-				changemmodesfornick(this, temp_nick, params + 1);
+				nickchange(this, temp_nick, params);
+				changemmodesfornick(this, temp_nick, params);
 			}
 		}
 		return;
@@ -936,99 +891,95 @@ void parseirc(char *serveroutput)
 	if(!strcasecmp(command, "KILL")) {
 		if(isme(params)) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(KILL)->I was killed by %s path %s\n", prefix + 1, restptr + 1);
+			oer_debug(OER_DEBUG_INFO, "parseirc(KILL)->I was killed by %s path %s\n", prefix, trailing);
 #endif
 		}
 		return;
 	}
 	if(!strcasecmp(command, "KICK")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
-		strncpy(channel, params, CHANLEN);
-		if((this = getchptr(channel)) == NULL) {
+		strncpy(userhost, prefix + tpos, USERHOSTLEN);
+		if((this = getchptr(params)) == NULL) {
 			return;
 		}
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", target, STRINGLEN, 0)) < 0) {
-			return;
-		}
-		if(isme(target)) {
+		if(isme(trailing)) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->I was kicked off %s by %s!%s\n", channel, temp_nick, userhost);
+			oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->I was kicked off %s by %s!%s\n", this->name, temp_nick, userhost);
 #endif
 			initchannel(this);
 			this->rejoin_at = mystate->now + OER_DELAY_BETWEEN_REJOINS;
 			return;
 		}
 		/* KICK doesn't tell the host for the person that was kicked */
-		if((cu = getcuptr(this, target)) == NULL) {
+		if((cu = getcuptr(this, trailing)) == NULL) {
                         /* we don't know the host, this is necessary because of
                            bogus/lagged server/service messages */
                         return;
                 }
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->%s!%s off %s by %s!%s\n", target, (cu->userhost == NULL) ? "(null)" : cu->userhost, channel, temp_nick, userhost);
+		oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->%s!%s off %s by %s!%s\n", trailing, (cu->userhost == NULL) ? "(null)" : cu->userhost, this->name, temp_nick, userhost);
 #endif
-		addnewpart(this, mystate->now, target, cu->userhost);
-		if(!userleft(this, target, cu->userhost)) {
+		addnewpart(this, mystate->now, trailing, cu->userhost);
+		if(!userleft(this, trailing, cu->userhost)) {
 #ifdef DEBUG
 			oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->userleft() returned NULL\n");
 #endif
 		}
-		initmmodesfornick(this, target);
+		initmmodesfornick(this, trailing);
 		if(this->nickcount == 1 && !this->i_am_op) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->cycling %s to gain ops\n", channel);
+			oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->cycling %s to gain ops\n", this->name);
 #endif
-			snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s", channel);
+			snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s", this->name);
 			timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, timed_str);
 		}
 		listchanusers();
 		return;
 	}
 	if(!strcasecmp(command, "NOTICE")) {
+		tpos = 0;
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+			return;
+		}
+		strncpy(userhost, prefix + tpos, USERHOSTLEN);
+		parsenotice(temp_nick, userhost, params, trailing);
 		return;
 	}
 	if(!strcasecmp(command, "MODE")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		if(strlen(prefix + 1 + tpos)) {
-			strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
+		if(strlen(prefix + tpos)) {
+			strncpy(userhost, prefix + tpos, USERHOSTLEN);
 		} else {
 			memset(userhost, 0, USERHOSTLEN + 1);
 		}
-		strncpy(channel, params, CHANLEN);
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", mode, CHANLEN, 0)) < 0) {
-			return;
-		}
-		if(mode[0] == ':') {
-			/* our usermode */
+		if(isme(params)) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->usermode is %s\n", mode + 1);
+			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->usermode is %s\n", trailing);
 #endif
 			return;
 		}
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(params)) == NULL) {
 			return;
 		}
+		restpos = 0;
 		/* channel modes have no target */
-		if(strlen(restptr + restpos) > 0) {
+		if(strlen(restptr)) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s %s on %s by %s!%s\n", mode, restptr + restpos, channel, temp_nick, userhost);
+			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s %s on %s by %s!%s\n", trailing, restptr, this->name, temp_nick, userhost);
 #endif
 		} else {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on %s by %s!%s\n", mode, channel, temp_nick, userhost);
+			oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on %s by %s!%s\n", trailing, this->name, temp_nick, userhost);
 #endif
 		}
 		for(i = 0, gotallmodes = 0, bans = 0, deops = 0, do_syncbans = 0; !gotallmodes; i++) {
-			switch((modetype = nthmode(mode, i))) {
+			switch((modetype = nthmode(trailing, i))) {
 			case OER_NTHMODE_OP:
 			case OER_NTHMODE_DOP:
 			case OER_NTHMODE_VOICE:
@@ -1037,22 +988,60 @@ void parseirc(char *serveroutput)
 			case OER_NTHMODE_UNBAN:
 			case OER_NTHMODE_CHANMODE_WITH_PARAMS:
 				nrestpos = parse(restptr, restpos, " ", target, STRINGLEN, 0);
-				if(nrestpos != restpos && nrestpos > 0) {
+				if((nrestpos != restpos) && (nrestpos > 0)) {
 					restpos = nrestpos;
-					getnthmode(mode, i + 1, modeline);
-					if(!strcasecmp(modeline, "+k")) {
-						strncpy(this->key, target, CHANLEN);
-					}
-					if(!strcasecmp(modeline, "-k")) {
-						memset(this->key, 0, CHANLEN + 1);
+					getnthmode(trailing, i + 1, modeline);
+					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->modeline is %s\n", modeline);
+					if(!isme(temp_nick)) {
+						/* the following are only necessary for
+						   manual user actions, check channel key */
+						if(!strcasecmp(modeline, "+k")) {
+							strncpy(this->key, target, CHANLEN);
+							if(getmodepol(this->mode, 'k') != '-') {
+								/* avoid 467 */
+								this->chankeyisset = 1;
+							}
+						} else if(!strcasecmp(modeline, "-k")) {
+							if(getmodepol(this->mode, 'k') != '+') {
+								memset(this->key, 0, CHANLEN + 1);
+							} else {
+								/* reset */
+								this->chankeyisset = 0;
+							}
+						}
+						/* check channel limit (+l) */
+						if(!strcasecmp(modeline, "+l")) {
+							strncpy(this->limit, target, CHANLEN);
+							if(getmodepol(this->mode, 'l') != '-') {
+								/* avoid re-setting */
+								this->chanlimitisset = 1;
+							} else {
+								this->chanlimitisset = 0;
+							}
+						}
 					}
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->target is %s\n", target);
+					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->target is %s\n", target);
 #endif
 				} else {
 					gotallmodes = 1;
 				}
 				break;
+			case OER_NTHMODE_CHANMODE_WITHOUT_PARAMS:
+				getnthmode(trailing, i + 1, modeline);
+				oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->modeline is %s\n", modeline);
+				if(!isme(temp_nick)) {
+					/* check channel limit (-l) */
+					if(!strcasecmp(modeline, "-l")) {
+						if(getmodepol(this->mode, 'l') != '+') {
+							memset(this->limit, 0, CHANLEN + 1);
+						} else {
+							/* reset */
+							this->chanlimitisset = 0;
+						}
+					}
+				}
+				break;
 			case OER_NTHMODE_UNKNOWN:
 				gotallmodes = 1;
 				break;
@@ -1074,12 +1063,12 @@ void parseirc(char *serveroutput)
 					continue;
 				}
 				if(haschanflags(this, "u") && !isallowedop(this, target)) {
-					if(isq(temp_nick) || isservice(temp_nick)) {
-						/* allow Q/service to op */
+					if(isq(temp_nick) || isservice(temp_nick) || hasuserflags(this, userhost, "s", "")) {
+						/* allow Q/service/special user to op */
 						continue;
 					}
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, temp_nick, userhost);
+					oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, this->name, temp_nick, userhost);
 #endif
 					mmode_new(this, mystate->now, "-o", target);
 				}
@@ -1101,8 +1090,8 @@ void parseirc(char *serveroutput)
 				if(isme(target)) {
 					this->i_am_op = 0;
 				}
-				if(isq(temp_nick) || isservice(temp_nick)) {
-					/* allow Q/service to de-op */
+				if(isq(temp_nick) || isservice(temp_nick) || hasuserflags(this, userhost, "s", "")) {
+					/* allow Q/service/special user to de-op */
 					continue;
 				}
 				if(!this->i_am_op) {
@@ -1141,14 +1130,14 @@ void parseirc(char *serveroutput)
 #ifdef DEBUG
 					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->ban detected on me (%s)\n", target);
 #endif
-					if(isq(temp_nick) || isservice(temp_nick)) {
-						/* allow Q/service to ban us */
+					if(isq(temp_nick) || isservice(temp_nick) || hasuserflags(this, userhost, "s", "")) {
+						/* allow Q/service/special user to ban us */
 						continue;
 					}
 					if(this->i_am_op && haschanflags(this, "b")) {
 						/* i am being banned, first kick then unban */
 						if(!isme(temp_nick) && !isserver(temp_nick)) {
-							snprintf(timed_str, WRITE_BUFFER_LENGTH, "KICK %s %s :do NOT ban %s, that's me!", channel, temp_nick, target);
+							snprintf(timed_str, WRITE_BUFFER_LENGTH, "KICK %s %s :do NOT ban %s, that's me!", this->name, temp_nick, target);
 							timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, timed_str);
 						}
 						mmode_new(this, mystate->now, "-b", target);
@@ -1162,13 +1151,13 @@ void parseirc(char *serveroutput)
 #endif
 				delchannelban(this, target);
 				if(this->i_am_op && haschanflags(this, "e") && ispermban(this, target)) {
-					if(isq(temp_nick) || isservice(temp_nick)) {
-						/* allow Q/service to unban */
+					if(isq(temp_nick) || isservice(temp_nick) || hasuserflags(this, userhost, "s", "")) {
+						/* allow Q/service/special user to unban */
 						continue;
 					}
 					/* re-set the ban */
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on channel %s tried to remove the permban %s\n", temp_nick, channel, target);
+					oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->%s on channel %s tried to remove the permban %s\n", temp_nick, this->name, target);
 #endif
 					mmode_new(this, mystate->now, "+b", target);
 					do_syncbans = 1;
@@ -1196,10 +1185,10 @@ void parseirc(char *serveroutput)
 			}
 			if((deops > OER_MASSMODE_LIMIT) && (index(userhost, '@') != NULL) && !isme(temp_nick)) {
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->mass modes detected on %s from %s\n", channel, temp_nick);
+				oer_debug(OER_DEBUG_INFO, "parseirc(MODE)->mass modes detected on %s from %s\n", this->name, temp_nick);
 #endif
-				if(isq(temp_nick) || isservice(temp_nick)) {
-					/* allow Q/service to do massmodes */
+				if(isq(temp_nick) || isservice(temp_nick) || hasuserflags(this, userhost, "s", "")) {
+					/* allow Q/service/special user to do massmodes */
 					continue;
 				}
 				if(this->i_am_op && haschanflags(this, "m") && !isserver(temp_nick)) {
@@ -1220,11 +1209,10 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "TOPIC")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(channel, params, CHANLEN);
-		if((this = getchptr(channel)) == NULL) {
+		if((this = getchptr(params)) == NULL) {
 			return;
 		}
                 if(haschanflags(this, "T")) {
@@ -1241,16 +1229,15 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "INVITE")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+		if((tpos = parse(prefix, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
-		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
+		strncpy(userhost, prefix + tpos, USERHOSTLEN);
+		if((this = getchptr(trailing)) == NULL) {
 			return;
 		}
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(INVITE)->%s invited me to %s\n", temp_nick, channel + 1);
+		oer_debug(OER_DEBUG_INFO, "parseirc(INVITE)->%s invited me to %s\n", temp_nick, this->name);
 #endif
 		return;
 	}
@@ -2661,3 +2648,29 @@ int checkauth(struct channel *this, char *userhost, char *nick, int tochan, int
 	}
 	return proceed;
 }
+
+void parsenotice(char *nick, char *userhost, char *target, char *message)
+{
+	struct channel * this;
+#ifdef DEBUG
+        oer_debug(OER_DEBUG_INFO, "parsenotice(\"%s\", \"%s\", \"%s\", \"%s\")\n", nick, userhost, target, message);
+#endif
+	/* check whether the notice is sent to a channel or not */
+	if((this = getchptr(target)) == NULL) {
+		return;
+	}
+	if(this->i_am_op && checkforansi(this, nick, userhost, message)) {
+		return;
+	}
+	if(this->i_am_op && checkforflood(this, nick, userhost, message)) {
+		return;
+	}
+	if(this->i_am_op && checkforbadword(this, nick, userhost, message)) {
+		return;
+	}
+	/* only add this consequetive line iff the user
+	   isn't being kicked already (lag between kick and last message) */
+	if(!gettobek(this, nick)) {
+		addnewpubmsg(this, mystate->now, nick, userhost, message);
+	}
+}
